#ifndef CONTENT_ACTION_ENGINE_H
#define CONTENT_ACTION_ENGINE_H

#include <QObject>
#include <QString>
#include <QStringList>

class ContentActionEnginePrivate;
class QAbstractItemModel;

/**
 * @brief API for querying actions.
 * @ingroup RelevanceAPI
 * @credential relevance::RelevanceAllContentTypes Needed for finding results
 * @credential smartsearch::RelevanceAllContentTypes Needed for finding results
 *
 * This class is used to query available actions for content items.
 **/
class ContentActionEngine : public QObject {

    Q_OBJECT

public:

    /**
     * Creates the content action engine with the given @p parent. Does not perform any actions yet.
     *
     * @param parent optional parent object.
     **/
    explicit ContentActionEngine (QObject *parent = 0);

    /**
     * Destructor.
     **/
    ~ContentActionEngine ();

    /**
     * All roles return a QString.
     **/
    enum ActionItemDataRole {
        Identifier = Qt::UserRole,
        Name,
        LocalizedName, // same as Qt::DisplayRole
#ifdef NETWORK_SEARCH
        Icon
#endif
    };

    /**
     * Query available actions for items
     *
     * @param identifier item identifier
     **/
    QAbstractItemModel* actionsForItem(const QString& identifier);

    /**
     * Query available actions for items
     *
     * @param identifiers item identifiers
     **/
    QAbstractItemModel* actionsForItems(const QStringList& identifiers);

    /**
     * Query available actions for items
     *
     * @param mimetype mime type identifier
     **/
    QAbstractItemModel* actionsForMimeType(const QString& mimetype);

    /**
     * Execute action for a item
     *
     * @param actionIdentifier action
     * @param itemIdentifier item
     **/
    void executeItem(const QString& actionIdentifier, const QString& itemIdentifier);

    /**
     * Execute default action for a item
     *
     * @param itemIdentifier item
     **/
    void executeItemDefault(const QString& itemIdentifier);

    /**
     * Execute action for items
     *
     * @param actionIdentifier action
     * @param itemIdentifier item
     **/
    void executeItems(const QString& actionIdentifier, const QStringList& itemIdentifiers);

private:

    //! private implementation and data
    ContentActionEnginePrivate * m_dp;
};

#endif // CONTENT_ACTION_ENGINE_H
