#ifndef CONTENT_MODEL_H
#define CONTENT_MODEL_H

#include <QAbstractItemModel>

/**
 * @brief Abstract base class for all content models handled by the content list.
 * @ingroup RelevanceAPI
 *
 * This is an abstract base class for content models that can feed content to the content list. It is
 * basically just a shallow interface on top of a QAbstractItemModel and adds some needed virtual methods.
 **/
class ContentModel : public QAbstractItemModel {

    Q_OBJECT

public:

    /**
     * Creates the model with the optional given @p parent.
     *
     * @param parent owner object.
     **/
    ContentModel (QObject * parent=0);

    /**
     * Pure virtual method for getting the primary text for an item. This text will be shown most prominently and should
     * contain the most important data according to the UI spec. The @p row indicates the index of the wanted item's text.
     *
     * @param row index of wanted item.
     *
     * @return the text to be shown as the primary text.
     **/
    virtual const QString getPrimaryText (int row) = 0;

    /**
     * Pure virtual method for getting the secondary text for an item. This text will be shown less prominently than the primary
     * text and should contain data according to the UI spec. The @p row indicates the index of the wanted item's text.
     *
     * @param row index of wanted item.
     *
     * @return the text to be shown as the secondary text.
     **/
    virtual const QString getSecondaryText (int row) = 0;

    /**
     * Pure virtual method for getting the primary icon for an item. This is shown on the left of the content item and is the "large icon"
     * for an item. Each item should have a primary pixmap. Usage according to the UI spec. The @p row indicates the index of the wanted item's icon.
     *
     * @param row index of wanted item.
     *
     * @return the icon to be shown as the primary icon.
     **/
    virtual const QPixmap getPrimaryImage (int row) = 0;

    /**
     * Pure virtual method for getting the secondary icon for an item. This is a smaller optional icon and is likely not used by all
     * content types. Usage according to the UI spec. The @p row indicates the index of the wanted item's icon..
     *
     * @param row index of wanted item.
     *
     * @return the icon to be shown as the secondary icon.
     **/
    virtual const QPixmap getSecondaryImage (int row) = 0;

    /**
     * Pure virtual method for getting the tertiary icon for an item. This is a smaller optional icon and is used for favorite status.
     * Other usage according to the UI spec. The @p row indicates the index of the wanted item's icon..
     *
     * @param row index of wanted item.
     *
     * @return the icon to be shown as the tertiary icon.
     **/
    virtual const QPixmap getTertiaryImage (int row) = 0;
    
    /**
     * Pure virtual method for getting the text of tertiary label. It's on the right-bottom of the content item.
     * Other usage according to the UI spec. The @p row indicates the index of the wanted. for example. for music item, it return the duration
     *
     * @param row index of wanted item.
     *
     * @return the string to be shown as the tertiary label
     **/
    virtual const QString getTertiaryLabelText (int row) = 0;


signals:

    /**
     * Signal that subclasses should emit when the model is ready to be used and all data has been fetched.
     **/
    void modelUpdated ();
};

#endif // CONTENT_MODEL_H
