#ifndef CONTENT_SEARCH_ENGINE_H
#define CONTENT_SEARCH_ENGINE_H

#include <QObject>
#include <QStringList>
#include <QDateTime>

class ContentModel;
class ContentSearchEnginePriv;
class QAbstractItemModel;

/**
 * @brief API for performing searches.
 * @ingroup RelevanceAPI
 * @credential relevance::RelevanceAllContentTypes Needed for finding results
 * @credential smartsearch::RelevanceAllContentTypes Needed for finding results
 *
 * This class is used to perform searches using all the known data providers and by taking given
 * search criteria into account. Searches can also be done without any criteria and then the most
 * relevant data is returned.
 *
 * @see ContentModel
 **/
class ContentSearchEngine : public QObject {

    Q_OBJECT

public:

    /**
     * Creates the content search engine with the given @p parent. Does not perform any searches yet.
     *
     * @param parent optional parent object.
     **/
    explicit ContentSearchEngine (QObject *parent = 0);

    /**
     * Destructor.
     **/
    ~ContentSearchEngine ();

    /**
     * Performs a search against all known enabled data providers with the given data. If no special parameters are given (default values)
     * then a set of the most relevant content from all providers will be created. The returned model will emit modelUpdated() when
     * all content has actually been retrieved and is ready to be used. The ownership of the model is transferred to the caller, this
     * class does not refer to it in any way anymore. If @p content_types contains content type filters then only the providers that support this
     * content type will be queried.
     *
     * If @p max_items is != -1 then that size is used as a maximum number of items to fetch of each content type. It's not a total amount of items,
     * but per content type.
     *
     * If the @p text refers to an emergency contact search then a result model that only contains the emergency contact will be returned.
     *
     * @param text a search string that is used to perform an initial search.
     * @param tags a list of fully qualified tag uris.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param properties a list of fully qualified or short property uris that list properties that the content must have.
     * @param start start date for a time limiting filter. This is normally the current date and time.
     * @param end the ending date and time.
     * @param max_items maximum number of items to fetch per content type.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    ContentModel * search (const QString & text="", const QStringList & tags=QStringList(), const QStringList & content_types=QStringList(),
                           const QStringList & properties=QStringList(), const QDateTime & start=QDateTime(),
                           const QDateTime & end=QDateTime(), int max_items=-1);

    /**
     * Performs a search for the contenty with the given @p uris and returns an allocated content model. This version does not use any other search critera
     * than the @p uris.
     *
     * @param uris a list of uris to the wanted content.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    ContentModel * search (const QStringList & uris);

    /**
     * Performs a group browse query
     *
     * @param group a group identifier.
     * @param text a search string that is used to perform an initial search.
     * @param tags a list of fully qualified tag uris.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param properties a list of fully qualified or short property uris that list properties that the content must have.
     * @param start start date for a time limiting filter. This is normally the current date and time.
     * @param end the ending date and time.
     * @param max_items maximum number of items to fetch per content type.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    ContentModel * browse (const QString& group, const QString & text="", const QStringList & tags=QStringList(),
                           const QStringList & content_types=QStringList(), const QStringList & properties=QStringList(),
                           const QDateTime & start=QDateTime(), const QDateTime & end=QDateTime(), int max_items=-1);

    // new API (still under construction)

    /**
     * All roles return a QString.
     * About images:
     *
     * @li file URL to a file in file system, e.g. "/home/user/.thumbnails/aaa.png"
     * @li theme icon to be fetched from the theme system, e.g. "[icon_id]"
     * @li plain text to be rendered as image, e.g. "12:30"
     * @li empty string "" means no image for that kind
     **/
    enum ResultItemDataRole {
        PrimaryLabel = Qt::DisplayRole, //<! the main text label for the item
        SecondaryLabel = Qt::UserRole,  //<! an optional secondary label
        PrimaryImage,                   //<! main image for the item. if this
        SecondaryImage,                 //<! an optional smaller secondary image
        TertiaryImage,                  //<! an optional smaller tertiary image
        Identifier,                     //<! the uri/url of the item
        GroupIdentifier,                //<! TODO: what is this?
        ContentType,                    //<! uri of a content type for the item
        ContentSource,                  //<! TODO: what is this?
        MimeType,                       //<! a mime type for the item
        PrimaryImagePlaceholder,        //<! a placeholder image name to be used until the real primary image is available
        TertiaryLabel,                  //<! optional tertiary label
        Details,                         //<! an optional mapping of item details
        IsGroup                        //<! item is a group 
    };

    //! Person Contact Type Identifier
    static const QString PersonContactType;

    //! Document Type Identifier
    static const QString DocumentType;

    //! Photo Type Identifier
    static const QString ImageType;

    //! Software Application Type Identifier
    static const QString SoftwareApplicationType;

    //! Web History Type Identifier
    static const QString WebHistoryType;

    //! Bookmark Type Identifier
    static const QString BookmarkType;

    //! Email Type Identifier
    static const QString EmailType;

    //! IM Message Type Identifier
    static const QString IMMessageType;

    //! SMS Message Type Identifier
    static const QString SMSMessageType;

    //! Event Type Identifier
    static const QString EventType;

    //! Todo Type Identifier
    static const QString TodoType;

    //! Journal Type Identifier
    static const QString JournalType;

    //! Feed Message Type Identifier
    static const QString FeedMessageType;

    //! Music Piece Type Identifier
    static const QString MusicPieceType;

    //! Artist Type Identifier
    static const QString ArtistType;

    //! Music Album Type Identifier
    static const QString MusicAlbumType;

    //! Video Type Identifier
    static const QString VideoType;

    //! Radio Station Type Identifier
    static const QString RadioStationType;

    //! File Data Object Type Identifier
    static const QString FileDataObjectType;

    //! Folder Type Identifier
    static const QString FolderType;

    //! Help Document Type Identifier
    static const QString HelpDocumentType;

    //! Tracker Source Identifier
    static const QString TrackerSource;

    //! default content types
    static const QStringList m_defaultTypes ;

    /**
     * Performs a search query
     *
     * This method supports returning groups.
     *
     * @param text a search string that is used to perform an initial search.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param content_sources a list of fully qualified or short content source uris that narrow down the available content.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    QAbstractItemModel* doSearch (const QString& text="", const QStringList& content_types=QStringList(),
                                  const QStringList& content_sources=QStringList());

    /**
     * Performs a search query
     *
     * This method does not return groups.
     *
     * @param text a search string that is used to perform an initial search.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param content_sources a list of fully qualified or short content source uris that narrow down the available content.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    QAbstractItemModel* doSearchWithoutGrouping (const QString& text="", const QStringList& content_types=QStringList(),
                                                 const QStringList& content_sources=QStringList());

    /**
     * Performs a group browse query
     *
     * This method supports returning groups.
     *
     * @param groupIdentifier a group identifier.
     * @param text a search string that is used to perform an initial search.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param content_sources a list of fully qualified or short content source uris that narrow down the available content.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    QAbstractItemModel* doSearchGroup (const QString& groupIdentifier, const QString& text="", const QStringList& content_types=QStringList(),
                                       const QStringList& content_sources=QStringList());

    /**
     * Performs a group browser query
     *
     * This method does not return groups.
     *
     * @param groupIdentifier a group identifier.
     * @param text a search string that is used to perform an initial search.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param content_sources a list of fully qualified or short content source uris that narrow down the available content.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    QAbstractItemModel* doSearchGroupWithoutGrouping (const QString& groupIdentifier, const QString& text="", const QStringList& content_types=QStringList(),
                                                      const QStringList& content_sources=QStringList());

    /**
     * Performs a search query
     *
     * @param identifiers a list of URIs to be searched.
     * @param content_types a list of fully qualified or short content type uris that narrow down the available content.
     * @param content_sources a list of fully qualified or short content source uris that narrow down the available content.
     *
     * @return a newly allocated model whose ownership is moved to the caller or 0 on an critical error.
     **/
    QAbstractItemModel* doSearchItems (const QStringList& identifiers, const QStringList& content_types=QStringList(),
                                       const QStringList& content_sources=QStringList());

private:

    //! private implementation and data
    ContentSearchEnginePriv * m_dp;
};

#endif // CONTENT_SEARCH_ENGINE_H
